#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import aiohttp
import asyncio
import jmespath
import json
import logging
import requests
import signal

from .providers import providers

logging.basicConfig(level=logging.WARNING)
logging.getLogger("requests").setLevel(logging.WARNING)

class ActivityCount(object):
  """Gather activity/share stats from social APIs"""
  def __init__(self, url=None):
    self.url = url or None
    self.responses = []

  def establish_client(self, loop):
    self.loop = asyncio.new_event_loop()
    self.client = aiohttp.ClientSession(loop=self.loop)
    asyncio.set_event_loop(self.loop)

  async def async_get_all(self, loop):
    self.establish_client(loop)
    
    for provider in providers:
      url = provider["endpoint"].format(self.url)
      asyncio.ensure_future(self.collect_sharecount(url, provider), loop=self.loop)

    # loop over all providers
    pending = asyncio.Task.all_tasks(loop=self.loop)
    self.loop.run_until_complete(asyncio.gather(*pending, loop=self.loop))
    self.client.close()
    self.loop.close()

  async def get_json(self, url):
    async with self.client.get(url) as response:
      assert response.status == 200
      logging.debug("Got response for URL {0} with statuscode {1}".format(url, response.status))
      response = await response.read()
      return response.decode('utf-8')

  async def collect_sharecount(self, url, provider):
    response = await self.get_json(url)
    j = json.loads(response)

    data = {
      "provider": provider["provider"],
      "metrics": []
    }

    for m in provider["metrics"]:
      data["metrics"].append({
        "count": jmespath.search(m["path"], j),
        "label": m["label"]
      })
    
    self.responses.append(data)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import itertools
from .blacklists import blacklists

def check_credibility(url):
  plain_lists = [l for l in list(blacklists.values())]
  consolidated_list = list(itertools.chain.from_iterable(plain_lists))
  confidence = consolidated_list.count(url) / len(blacklists)
  unique_set = set(consolidated_list)
  
  return {
    "is_blacklisted": url in consolidated_list,
    "fake_confidence": "{0:.2f}".format(confidence)
  }


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import difflib
import operator
import nltk
import numpy
import string
import re

from nltk.tokenize import RegexpTokenizer
from .pos import AveragedPerceptronTagger

tokenizer = RegexpTokenizer(r'\w+')

def isPunct(word):
  pattern = r"(`|\.|#|\$|%|&|\'|\(|\)|\*|\||\+|,|-|—|/|:|;|<|=|>|\?|@|\[|\]|\^|_|`|{|}|~|”|“|’)"
  return re.search(pattern, word) is not None

class EntityExtractor(object):
  def __init__(self, text):
    self.perceptron_tagger = AveragedPerceptronTagger(autoload=True)
    self.stopwords = set(nltk.corpus.stopwords.words())
    self.top_fraction = 70 # consider top candidate keywords only
    self.sent_detector = nltk.data.load('tokenizers/punkt/english.pickle')
    self.sentences = sent_detector.tokenize(text)

  def _calculate_word_scores(self, word_list):
    """Quick and dirty, inspired by Sujit Pal's RAKE implementation.
    """
    word_freq = nltk.FreqDist()
    for word in word_list:
      word_freq[word] += 1
    
    word_scores = {k:v for k, v in word_freq.items() if v > 0}
    return word_scores

  # def _get_mt_median(self, word_scores):
  #   median = numpy.median([v for k, v in word_scores.items()])
  #   return {k: v for k, v in word_scores.items() if v > median}

  def _filter_distance(self, words):
    close_matches = []
    wordlist = set(words[:]) # deepcopy

    for word in words:
      if word in close_matches: continue
      matches = difflib.get_close_matches(word, wordlist, 2)
      if len(matches) > 1:
        close_matches += matches[1:]

    return wordlist.difference(close_matches)

  def _sort_and_filter(self, word_scores):
    n_words = len(word_scores)
    sorted_word_scores = sorted(word_scores.items(), key=operator.itemgetter(1), reverse=True)
    top_words = sorted_word_scores[0:int(n_words/100*self.top_fraction)]
    punct_filtered = [k[0] for k in top_words if not isPunct(k[0])]
    distance_filtered = self._filter_distance(punct_filtered)
    return list(distance_filtered)

  def _contains_stopword(self, ent):
    filtered = [word.lower() in self.stopwords for word in ent.split(" ")]
    return True in filtered

  def get_scored_entities(self):
    named_ents = []

    for sent in self.sentences:
      pos_tags = self.perceptron_tagger.tag(" ".join(nltk.word_tokenize(sent)))
      entities = self.perceptron_tagger.named_entities(pos_tags)
      named_ents += [ent for ent in entities if not self._contains_stopword(ent)]

    ent_scores = self._calculate_word_scores(named_ents)
    self.ent_scores = ent_scores
    return ent_scores

  def get_names(self):
    filtered_names = {k: v for k, v in self.ent_scores.items() if len(k.split(" ")) > 1}
    top_names = self._sort_and_filter(filtered_names)
    return top_names[:8]
    
  def get_keywords(self):
    filtered_keywords = {k.lower(): v for k, v in self.ent_scores.items() if len(k.split(" ")) == 1}
    top_keywords = self._sort_and_filter(filtered_keywords)
    return top_keywords[:8]



"""While we're only interested in share, like, up counts for now,
there's a lot of interesting metadata in e.g. reddit responses like
user_reports, report_reasons, num_reports, that might be useful
in building certain heuristics.
"""

providers = [
  {
    "provider": "facebook",
    "endpoint": "https://graph.facebook.com/?id={0}",
    "metrics": [{
      "label": "sharecount",
      "path": "share.share_count"
    }]
  },
  {
    "provider": "linkedin",
    "endpoint": "https://www.linkedin.com/countserv/count/share?url={0}/&format=json",
    "metrics": [{
      "label": "sharecount",
      "path": "count"
    }]
  },
  {
    "provider": "reddit",
    "endpoint": "https://buttons.reddit.com/button_info.json?url={0}",
    "metrics": [{
      "label": "upvotes",
      "path": "data.children[0].data.ups"
    },
    {
      "label": "num_reports",
      "path": "data.children[0].data.num_reports"
    }]
  }
]


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import asyncio
import math
import tldextract
from datetime import datetime, timedelta
from .lookup import whois_date_registered
from .check import check_credibility

class Domaintools(object):
  """Gather various metadata like whois informaion
  and blacklist status about any given hostname
  """
  def __init__(self, url=None):
    self.url = url or None
    self.get_domain(url)

  def get_domain(self, url):
    no_fetch_extract = tldextract.TLDExtract(suffix_list_urls=None)
    tld = no_fetch_extract(url)
    self.domain = "{}.{}".format(tld.domain, tld.suffix)

  def get_date_registered(self):
    self.date_registered = whois_date_registered(self.domain)

  def check_credibility(self):
    self.credibility = check_credibility(self.domain)

  def get_all(self):
    if not self.domain: return
    self.get_date_registered()
    self.check_credibility()
    
    if self.date_registered:
      self.recalculate_fake_confidence()
      self.date_registered_iso = self.date_registered.isoformat()

  def recalculate_fake_confidence(self):
    # Adds .2 to fake_confidence if website was registered delta 1y
    one_year_ago = datetime.now() - timedelta(days=1*365)
    if self.date_registered < one_year_ago: return

    confidence = self.credibility.get("fake_confidence", 0)
    self.credibility["fake_confidence"] = float(confidence) + .2

  async def async_get_all(self, loop):
    asyncio.set_event_loop(loop)
    return await loop.run_in_executor(None, self.get_all)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__title__ = 'Metadoc - Postmodern news article metadata service'
__copyright__ = 'Copyright 2016, Paul Solbach'
__author__ = 'Paul Solbach'
__license__ = 'MIT'

import asyncio
import concurrent
import requests

from .domain import Domaintools
from .extract import Extractor
from .social import ActivityCount

import logging
logging.basicConfig(level=logging.WARN)

class Metadoc(object):
  def __init__(self, url=None, html=None, **kwargs):
    """Metadoc API, initialize with
    :param url: The article url we shall investigate, required.
    :param html: You can pass in the article html manually, optional.
    """

    self.html = html or None
    self.url = url or None
    
    if not url:
      raise AttributeError('Missing \"url\" attribute.')

    if not self.html:
      self._request_url()

    self.extractor = Extractor(html=self.html) # Named entities, synthetic summaries
    self.activity = ActivityCount(url=self.url) # Social activity from various networks
    self.domain = Domaintools(url=self.url) # Domain whois date, blacklisting

  def query_all(self):
    """Combine all available resources"""
    subtasks = []

    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)

    for c in [self.activity, self.extractor, self.domain]:
      subtasks.append(c.async_get_all(loop))

    loop.run_until_complete(asyncio.wait(subtasks, loop=loop))
    loop.close()

  def return_ball(self):
    return self._render()

  def query_domain(self):
    self.domain.get_all()

  async def query_social(self):
    await self.activity.async_get_all(asyncio.get_event_loop())

  def query_extract(self):
    self.extractor.get_all()

  def _render(self):
    """Construct response dict after partial or complete
    queries to various sources
    """
    return {
      "url": self.url,
      "title": getattr(self.extractor, "title", None),
      "authors": getattr(self.extractor, "authors", None),
      "image": getattr(self.extractor, "image", None),
      "social": getattr(self.activity, "responses", None),
      "language": getattr(self.extractor, "language", None),

      "text": {
        "fulltext": getattr(self.extractor, "fulltext", None),
        "reading_time": getattr(self.extractor, "reading_time", None),
        "contenthash": getattr(self.extractor, "contenthash", None)
      },
      
      "entities": {
        "names": getattr(self.extractor, "names", None),
        "keywords": getattr(self.extractor, "keywords", None),
      },
      
      "domain": {
        "name": getattr(self.domain, "domain", None),
        "credibility": getattr(self.domain, "credibility", None),
        "date_registered": getattr(self.domain, "date_registered_iso", None),
        "favicon": "https://logo.clearbit.com/{0}?size=200".format(getattr(self.domain, "domain", None)),
      }
    }

  def _request_url(self):
    """In case no html parameter was provided to the constructor"""
    req = requests.get(self.url)
    if req.status_code != 200:
      raise Exception('Requesting article body \
        failed with {1} status code.'.format(req.status_code))

    self.html = req.text # after except



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Averaged perceptron classifier. Implementation geared for simplicity rather than
efficiency. Adapted from @hankcs, cf. https://github.com/hankcs/AveragedPerceptronPython/blob/master/LICENSE
Based on http://honnibal.wordpress.com/2013/09/11/a-good-part-of-speechpos-tagger-in-about-200-lines-of-python/
"""

from collections import defaultdict
import pickle
import random
import logging
import os

PICKLE = os.path.join(os.path.dirname(__file__), "data/tagger-0.3.5.pickle")
TRAINING_SET = os.path.join(os.path.dirname(__file__), "data/training_set.txt")

logging.basicConfig(level=logging.INFO)

def do_train():
  tagger = AveragedPerceptronTagger(autoload=False)
  logging.info('Reading corpus.')
  training_data = []
  sentence = ([], [])

  for line in open(TRAINING_SET):
    params = line.split(' ')
    if len(params) != 2: continue

    sentence[0].append(params[0])
    sentence[1].append(params[1])

    if params[0] == '.':
      training_data.append(sentence)
      sentence = ([], [])

  logging.info('training corpus size : %d', len(training_data))
  logging.info('Start training...')
  tagger.train(training_data, save_loc=PICKLE)

class AveragedPerceptron(object):
  '''An averaged perceptron, as implemented by Matthew Honnibal.
  '''

  def __init__(self):
    # Each feature gets its own weight vector, so weights is a dict-of-dicts
    self.weights = {}
    self.classes = set()
    # The accumulated values, for the averaging. These will be keyed by
    # feature/clas tuples
    self._totals = defaultdict(int)
    # The last time the feature was changed, for the averaging. Also
    # keyed by feature/clas tuples
    # (tstamps is short for timestamps)
    self._tstamps = defaultdict(int)
    # Number of instances seen
    self.i = 0

  def predict(self, features):
    '''Dot-product the features and current weights and return the best label.'''
    scores = defaultdict(float)
    for feat, value in features.items():
      if feat not in self.weights or value == 0:
        continue
      weights = self.weights[feat]
      for label, weight in weights.items():
        scores[label] += value * weight
    # Do a secondary alphabetic sort, for stability
    return max(self.classes, key=lambda label: (scores[label], label))

  def update(self, truth, guess, features):
    '''Update the feature weights.'''
    def upd_feat(c, f, w, v):
      param = (f, c)
      self._totals[param] += (self.i - self._tstamps[param]) * w
      self._tstamps[param] = self.i
      self.weights[f][c] = w + v

    self.i += 1
    if truth == guess:
      return None
    for f in features:
      weights = self.weights.setdefault(f, {})
      upd_feat(truth, f, weights.get(truth, 0.0), 1.0)
      upd_feat(guess, f, weights.get(guess, 0.0), -1.0)
    return None

  def average_weights(self):
    '''Average weights from all iterations.'''
    for feat, weights in self.weights.items():
      new_feat_weights = {}
      for clas, weight in weights.items():
        param = (feat, clas)
        total = self._totals[param]
        total += (self.i - self._tstamps[param]) * weight
        averaged = round(total / float(self.i), 3)
        if averaged:
          new_feat_weights[clas] = averaged
      self.weights[feat] = new_feat_weights
    return None

  # def save(self, path):
  #   '''Save the pickled model weights.'''
  #   return pickle.dump(dict(self.weights), open(path, 'w'))

  # def load(self, path):
  #   '''Load the pickled model weights.'''
  #   self.weights = pickle.load(open(path))
  #   return None

class AveragedPerceptronTagger(object):
  '''Greedy Averaged Perceptron tagger, as implemented by Matthew Honnibal.
  :param load: Load the pickled model upon instantiation.
  '''

  START = ['-START-', '-START2-']
  END = ['-END-', '-END2-']
  AP_MODEL_LOC = PICKLE

  def __init__(self, autoload=False):
    self.model = AveragedPerceptron()
    self.tagdict = {}
    self.classes = set()

    if autoload:
      self.load(self.AP_MODEL_LOC)

  def tag(self, corpus):
    '''Tags a string `corpus`.'''
    # Assume untokenized corpus has \n between sentences and ' ' between words
    s_split = lambda t: t.split('\n')
    w_split = lambda s: s.split()

    def split_sents(corpus):
      for s in s_split(corpus):
        yield w_split(s)

    prev, prev2 = self.START
    tokens = []

    for words in split_sents(corpus):
      context = self.START + [self._normalize(w) for w in words] + self.END
      for i, word in enumerate(words):
        tag = self.tagdict.get(word)
        if not tag:
          features = self._get_features(i, word, context, prev, prev2)
          tag = self.model.predict(features)

        tokens.append((word, tag.strip()))
        prev2 = prev
        prev = tag

    return tokens

  def named_entities(self, tags):
    '''return sequential named entities,
    IO classification isn't as accurate here, since we're not differentiating between PERSON and ORGANIZATION.
    Still, this is fast and in many cases suited to the task.

    [('The', 'DT'), ('extraordinary', 'JJ'), ('phenomenon', 'NN'), ('of', 'IN'), ('fake', 'JJ'), 
    ('news', 'NN'), ('spread', 'NN'), ('by', 'IN'), ('Facebook', 'NNP'), ('and', ''), ('other', 'JJ'), 
    ('social', 'JJ'), ('media', 'NNS'), ('during', 'IN'), ('the', 'DT'), ('2016', 'CD'), ('presidential', 'JJ'), 
    ('election', 'NN'), ('has', 'VBZ'), ('been', 'VBN'), ('largely', 'RB'), ('portrayed', 'VBN'), ('as', 'IN'), 
    ('a', 'DT'), ('lucky', 'JJ'), ('break', 'NN'), ('for', 'IN'), ('Donald', 'NNP'), ('Trump', 'NNP')]
    '''

    ent, entities = [], []
    tags_len = len(tags)-1
    push_ent = lambda x: entities.append(" ".join(ent))

    for i, tag in enumerate(tags):
      if tag[1] == "NNP":
        ent.append(tag[0])
        if i == tags_len:
          push_ent(ent)

      elif len(ent):
        push_ent(ent)
        ent = []
    
    return entities

  def train(self, sentences, save_loc=None, nr_iter=5):
    '''Train a model from sentences, and save it at ``save_loc``. ``nr_iter``
    controls the number of Perceptron training iterations.
    :param sentences: A list of (words, tags) tuples.
    :param save_loc: If not ``None``, saves a pickled model in this location.
    :param nr_iter: Number of training iterations.
    '''
    self._make_tagdict(sentences)
    self.model.classes = self.classes
    for iter_ in range(nr_iter):
      c = 0
      n = 0
      for words, tags in sentences:
        prev, prev2 = self.START
        context = self.START + [self._normalize(w) for w in words] \
             + self.END
        for i, word in enumerate(words):
          guess = self.tagdict.get(word)
          if not guess:
            feats = self._get_features(i, word, context, prev, prev2)
            guess = self.model.predict(feats)
            self.model.update(tags[i], guess, feats)
          prev2 = prev
          prev = guess
          c += guess == tags[i]
          n += 1
      random.shuffle(sentences)
      logging.info("Iter {0}: {1}/{2}={3}".format(iter_, c, n, _pc(c, n)))
    self.model.average_weights()
    
    # Pickle as a binary file
    if save_loc is not None:
      pickle.dump((self.model.weights, self.tagdict, self.classes),
            open(save_loc, 'wb'), -1)
    
    return None

  def load(self, loc=None):
    '''Load a pickled model.'''
    try:
      w_td_c = pickle.load(open(loc, 'rb'))
    except IOError:
      raise IOError("Invalid perceptrontagger.pickle file.")

    self.model.weights, self.tagdict, self.classes = w_td_c
    self.model.classes = self.classes
    return None

  def _normalize(self, word):
    '''Normalization used in pre-processing.
    - All words are lower cased
    - Digits in the range 1800-2100 are represented as !YEAR;
    - Other digits are represented as !DIGITS
    :rtype: str
    '''
    if '-' in word and word[0] != '-':
      return '!HYPHEN'
    elif word.isdigit() and len(word) == 4:
      return '!YEAR'
    elif word[0].isdigit():
      return '!DIGITS'
    else:
      return word.lower()

  def _get_features(self, i, word, context, prev, prev2):
    '''Map tokens into a feature representation, implemented as a
    {hashable: float} dict. If the features change, a new model must be
    trained.
    '''

    def add(name, *args):
      features[' '.join((name,) + tuple(args))] += 1

    i += len(self.START)
    features = defaultdict(int)
    # It's useful to have a constant feature, which acts sort of like a prior
    add('bias')
    add('i suffix', word[-3:])
    add('i pref1', word[0])
    add('i-1 tag', prev)
    add('i-2 tag', prev2)
    add('i tag+i-2 tag', prev, prev2)
    add('i word', context[i])
    add('i-1 tag+i word', prev, context[i])
    add('i-1 word', context[i - 1])
    add('i-1 suffix', context[i - 1][-3:])
    add('i-2 word', context[i - 2])
    add('i+1 word', context[i + 1])
    add('i+1 suffix', context[i + 1][-3:])
    add('i+2 word', context[i + 2])
    return features

  def _make_tagdict(self, sentences):
    '''Make a tag dictionary for single-tag words.'''
    counts = defaultdict(lambda: defaultdict(int))

    for words, tags in sentences:
      for word, tag in zip(words, tags):
        counts[word][tag] += 1
        self.classes.add(tag)
    freq_thresh = 20
    ambiguity_thresh = 0.97

    for word, tag_freqs in counts.items():
      tag, mode = max(tag_freqs.items(), key=lambda item: item[1])
      n = sum(tag_freqs.values())
      # Don't add rare words to the tag dictionary
      # Only add quite unambiguous words
      if n >= freq_thresh and (float(mode) / n) >= ambiguity_thresh:
        self.tagdict[word] = tag


def _pc(n, d):
  return (float(n) / d) * 100



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import asyncio
import lxml
import nltk
import math
import hashlib

from langdetect import detect
from libextract.api import extract

from .ner import EntityExtractor
from .html import HtmlMeta

class Extractor(object):
  """Entity recognition, pullquote extraction etc.
  """
  def __init__(self, html=None, title=" ", **kwargs):
    self.html = html or None
    self.title = title or None
    self.entities = []

  def detect_language(self):
    """Langdetect is non-deterministic, so to achieve a higher probability
    we attempt detection multiple times and only report success if we get identical results.
    """
    nondet_attempts = [detect(self.fulltext) for i in range(0,2)]
    is_unique = len(set(nondet_attempts)) == 1
    self.language = nondet_attempts[0] if is_unique else False

  def sanitize_html(self):
    # Lxml bails out on html w/ emojis
    emoji_pattern = re.compile("["
        u"\U0001F600-\U0001F64F"  # emoticons
        u"\U0001F300-\U0001F5FF"  # symbols & pictographs
        u"\U0001F680-\U0001F6FF"  # transport & map symbols
        u"\U0001F1E0-\U0001F1FF"  # flags (iOS)
      "]+", flags=re.UNICODE)
  
    self.html = emoji_pattern.sub(r'', self.html)

  def extract_text(self):
    """Parse fulltext, do keyword extraction using the newspaper lib
    => newspaper.readthedocs.io
    """
    libextract_nodes = list(extract(self.html.encode("utf-8")))
    self.fulltext = libextract_nodes[0].text_content()

    entities = EntityExtractor(self.fulltext)
    entities.get_scored_entities() # Averaged Perceptron Tagger
    self.keywords = entities.get_keywords() # Above median?
    self.names = entities.get_names() # Filter top

  def extract_metadata(self):
    """Sniff for essential and additional metadata via
    either metatags and or json-ld"""
    html_meta = HtmlMeta(self.html)
    html_meta.extract()

    self.authors = html_meta.jsonld.get("authors") \
      or html_meta.metatags.get("article:author") \
      or html_meta.metatags.get("author")

    self.title = html_meta.jsonld.get("headline") or html_meta.title
    self.image = html_meta.metatags.get("twitter:image") or html_meta.jsonld.get("thumbnailUrl")

  def get_contenthash(self):
    """Generate md5 hash over title and body copy in order to keep track
    of changes made to a text, do diffs if necessary
    """
    contentstring = (self.title + self.fulltext).encode("utf-8")
    self.contenthash = hashlib.md5(contentstring).hexdigest()
    return self.contenthash

  def get_reading_time(self):
    """Calculate average reading time in seconds"""
    if not self.fulltext: return None
    wordcount = len(self.fulltext.split())
    self.reading_time = math.floor(wordcount / 300 * 60)

  def get_all(self):
    self.sanitize_html()
    self.extract_text()
    self.extract_metadata()
    self.detect_language()
    self.get_contenthash()
    self.get_reading_time()
    return

  async def async_get_all(self, loop):
    asyncio.set_event_loop(loop)
    return await loop.run_in_executor(None, self.get_all)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

blacklists = {
  "propornot": [
    "4thmedia.org",
    "nsnbc.me",
    "presstv.com",
    "theunhivedmind.com",
    "sana.sy",
    "activistpost.com",
    "americasfreedomfighters.com",
    "beforeitsnews.com",
    "corbettreport.com",
    "drudgereport.com",
    "endingthefed.com",
    "globalresearch.ca",
    "hangthebankers.com",
    "infowars.com",
    "naturalnews.com",
    "paulcraigroberts.org",
    "ronpaulinstitute.org",
    "southfront.org",
    "theantimedia.org",
    "trueactivist.com",
    "veteranstoday.com",
    "washingtonsblog.com",
    "yournewswire.com",
    "zerohedge.com",
    "4threvolutionarywar.wordpress.com",
    "abeldanger.net",
    "ahtribune.com",
    "allnewspipeline.com",
    "americanlookout.com",
    "amren.com",
    "amtvmedia.com",
    "ancient-code.com",
    "anonews.co",
    "anonhq.com",
    "antiwar.com",
    "asia-pacificresearch.com",
    "assassinationscience.com",
    "baltimoregazette.com",
    "barenakedislam.com",
    "bignuggetnews.com",
    "blackagendareport.com",
    "blacklistednews.com",
    "christianfightback.com",
    "collective-evolution.com",
    "conservativedailypost.com",
    "consortiumnews.com",
    "cosmicscientist.com",
    "countercurrents.org",
    "counterinformation.wordpress.com",
    "dailyoccupation.com",
    "dailystormer.com",
    "darkmoon.me",
    "darkpolitricks.com",
    "davidstockmanscontracorner.com",
    "dcclothesline.com",
    "dcleaks.com",
    "defenddemocracy.press",
    "dennismichaellynch.com",
    "disclose.tv",
    "disclosuremedia.net",
    "educate-yourself.org",
    "educateinspirechange.org",
    "endoftheamericandream.com",
    "endtime.com",
    "eutopia.buzz",
    "ewao.com",
    "eyeopening.info",
    "fellowshipoftheminds.com",
    "filmsforaction.org",
    "floridasunpost.com",
    "foreignpolicyjournal.com",
    "fourwinds10.net",
    "freedomoutpost.com",
    "gaia.com",
    "galacticconnection.com",
    "gatesofvienna.net",
    "geopolmonitor.com",
    "godlikeproductions.com",
    "govtslaves.info",
    "greanvillepost.com",
    "guccifer2.wordpress.com",
    "healthnutnews.com",
    "henrymakow.com",
    "heresyblog.net",
    "humansarefree.com",
    "ihavethetruth.com",
    "ihavethetruth.com",
    "in5d.com",
    "informationclearinghouse.info",
    "intellihub.com",
    "intrepidreport.com",
    "investmentresearchdynamics.com",
    "investmentwatchblog.com",
    "jackpineradicals.com",
    "jamesrgrangerjr.com",
    "jewsnews.co.il",
    "journal-neo.org",
    "katehon.com",
    "katehon.org",
    "kingworldnews.com",
    "lewrockwell.com",
    "libertyblitzkrieg.com",
    "libertywritersnews.com",
    "makeamericagreattoday.com",
    "mintpressnews.com",
    "moonofalabama.org",
    "nakedcapitalism.com",
    "naturalblaze.com",
    "newcoldwar.org",
    "newstarget.com",
    "newswithviews.com",
    "nowtheendbegins.com",
    "off-guardian.org",
    "oftwominds.com",
    "opednews.com",
    "orientalreview.org",
    "patriotrising.com",
    "platosguns.com",
    "pravda.ru",
    "pravdareport.com",
    "prepperwebsite.com",
    "prisonplanet.com",
    "rbth.com",
    "readynutrition.com",
    "redflagnews.com",
    "regated.com",
    "rense.com",
    "righton.com",
    "rinf.com",
    "rt.com",
    "rumormillnews.com",
    "ruptly.tv",
    "russia-insider.com",
    "sentinelblog.com",
    "sgtreport.com",
    "shiftfrequency.com",
    "shtfplan.com",
    "silentmajoritypatriots.com",
    "silverdoctors.com",
    "sott.net",
    "sputniknews.com",
    "stormcloudsgathering.com",
    "strategic-culture.org",
    "superstation95.com",
    "survivopedia.com",
    "the-newspapers.com",
    "thecommonsenseshow.com",
    "thedailybell.com",
    "thedailysheeple.com",
    "theduran.com",
    "theearthchild.co.za",
    "theeconomiccollapseblog.com",
    "theeventchronicle.com",
    "thefederalistpapers.org",
    "thefreethoughtproject.com",
    "themindunleashed.org",
    "thenewsdoctors.com",
    "therebel.media",
    "therussophile.org",
    "thesaker.is",
    "thesleuthjournal.com",
    "thetruenews.info",
    "thetruthseeker.co.uk",
    "thirdworldtraveler.com",
    "toprightnews.com",
    "trunews.com",
    "truth-out.org",
    "truthandaction.org",
    "truthdig.com",
    "truthfeed.com",
    "truthkings.com",
    "ufoholic.com",
    "undergroundworldnews.com",
    "unz.com",
    "usanewshome.com",
    "usapoliticsnow.com",
    "usasupreme.com",
    "usdcrisis.com",
    "usslibertyveterans.org",
    "vdare.com",
    "veteransnewsnow.com",
    "vigilantcitizen.com",
    "viralliberty.com",
    "voltairenet.org",
    "wakeupthesheep.com",
    "wakingtimes.com",
    "wearechange.org",
    "weshapelife.org",
    "whatdoesitmean.com",
    "whatreallyhappened.com",
    "wikileaks.com",
    "wikileaks.org",
    "wikispooks.com",
    "worldnewspolitics.com",
    "worldpoliticsus.com",
    "www.fort-russ.com",
    "oilgeopolitics.net",
    "gangstergovernment.com",
    "memoryholeblog.com",
    "eutimes.net",
    "intersectionproject.eu"
  ],
  "fortliberty.org": [
    "21stcenturywire.com",
    "800whistleblower.com",
    "activistpost.com",
    "alternet.org",
    "americannews.com",
    "antiwar.com",
    "beforeitsnews.com",
    "bigpzone.com",
    "chronicle.su",
    "consciouslifenews.com",
    "conspiracywire.com",
    "countdowntozerotime.com",
    "counterpsyops.com",
    "dailybuzzlive.com",
    "dailycurrant.com",
    "dcclothesline.com",
    "disclose.tv",
    "duffelblog.com",
    "duhprogressive.com",
    "elitereaders.com",
    "empirenews.net",
    "english.ruvr.ru",
    "eutimes.net",
    "federalistpress.com",
    "freepatriot.org",
    "fromthetrenchesworldreport.com",
    "geoengineeringwatch.org",
    "globalresearch.ca",
    "gonzoglobe.com",
    "govtslaves.info",
    "guardianlv.com",
    "gulagbound.com",
    "hangthebankers.com",
    "healthimpactnews.com",
    "humansarefree.com",
    "huzlers.com",
    "infowars.com",
    "intellihub.com",
    "lewrockwell.com",
    "libertynews.com",
    "livefreelivenatural.com",
    "nationalreport.net",
    "naturalcuresnotmedicine.com",
    "naturalnews.com",
    "newswire-24.com",
    "nodisinfo.com",
    "notallowedto.com",
    "now8news.com",
    "nowtheendbegins.com",
    "pakalertpress.com",
    "politicalblindspot.com",
    "presstv.ir",
    "prisonplanet.com",
    "randpaulreview.com",
    "rawforbeauty.com",
    "realfarmacy.com",
    "redflagnews.com",
    "responsibletechnology.org",
    "rt.com",
    "secretsofthefed.com",
    "southweb.org",
    "thecommonsenseshow.com",
    "thecontroversialfiles.net",
    "thedailysheeple.com",
    "thefreethoughtproject",
    "thelastgreatstand.com",
    "thenewamerican.com",
    "theracketreport.com",
    "therightplanet.com",
    "therundownlive.com",
    "theuspatriot.com",
    "topinfopost.com",
    "truthandaction.org",
    "truthbroadcastnetwork.com",
    "turnerradionetwork.com",
    "undergroundhealth.com",
    "usahitman.com",
    "veteranstoday.com",
    "westernjournalism.com",
    "whydontyoutrythis.com",
    "worldnewsdailyreport.com",
    "worldtruth.tv",
    "yournewswire.com"
  ],
  "zimdar": [
    "enduringvision.com",
    "70news.wordpress.com",
    "abcnews.com.co",
    "politicalo.com",
    "americannews.com",
    "indecisionforever.com",
    "realnewsrightnow.com",
    "infowars.com",
    "rilenews.com",
    "civictribune.com",
    "mediamass.net",
    "megynkelly.us",
    "msnbc.com.co",
    "msnbc.website",
    "nationalreport.net",
    "creambmp.com",
    "news-hound.com",
    "newsbiscuit.com",
    "dcgazette.com",
    "politicops.com",
    "newsmutiny.com",
    "drudgereport.com.co",
    "empirenews.net"
  ]
}


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import urllib
import whois
import datetime

def whois_date_registered(domain):
  query = whois.query(domain) # silently fails in corporate env, vocally fails behind proxy
  # if query.creation_date == "before aug-1996": query.creation_date = datetime.datetime(1996) # .co.uk edge case
  # elif type(query.creation_date) is not "date": query = None
  return query.creation_date if query else None


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import sys
from subprocess import call
from setuptools import setup, find_packages
from setuptools.command.install import install as _install

version = '0.3.5'

def _post_install(dir):
  return
  # call([sys.executable, '__install__.py'],
  #   cwd=os.path.join(dir, 'metadoc'))

class CustomInstall(_install):
  """Do stuff after setup."""
  def run(self):
    _install.run(self)
    self.execute(_post_install, (self.install_lib,),
      msg="Running post install task")

setup(
  name='metadoc',
  version=version,
  description="Post-truth era news article metadata service.",
  long_description="",
  classifiers=[ # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    "Programming Language :: Python :: 3.5",
    "Topic :: Internet :: WWW/HTTP", 
    "Development Status :: 3 - Alpha",
    "License :: OSI Approved :: MIT License",
    "Intended Audience :: Developers",
    "Operating System :: POSIX :: Linux",
    "Environment :: Web Environment",
    ], 

  keywords=["scraping", "metadata", "news article"],
  author='Paul Solbach',
  author_email='p@psolbach.com',
  url='https://github.com/psolbach/metadoc',
  license='MIT',
  cmdclass={'install': CustomInstall},
  packages=find_packages(exclude=['tests']),
  include_package_data=True,
  zip_safe=False,
  install_requires=[
    'aiohttp==1.1.5',
    'asynctest==0.9.0',
    'bottle==0.12.10',
    'jmespath==0.9.0',
    'langdetect==1.0.7',
    'libextract==0.0.12',
    'nltk==3.2.1',
    'pytest==3.0.5',
    'pytest-cov==2.4.0',
    'numpy==1.11.2',
    'tldextract==2.0.2',
    'requests==2.12.2',
    'whois==0.7'
  ]
)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import nltk
import time

def install():
  REQUIRED_CORPORA = [
    'brown', # Required for FastNPExtractor
    'punkt', # Required for WordTokenizer
    'wordnet', # Required for lemmatization and Wordnet
    'maxent_ne_chunker',
    'stopwords',
    'words' 
  ]

  for each in REQUIRED_CORPORA:
    print(('[+] Downloading corpus:  "{0}"'.format(each)))
    nltk.download(each)

  from metadoc.extract.pos import do_train
  print('[+] Training tagger now.')
  do_train()

if __name__ == "__main__":
  install()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import lxml.etree, lxml.html
from collections import ChainMap

class HtmlMeta(object):
  """Extract metadata from html.
  Needs work, e.g. handling multiple @property=author tags,
  detect if author content is a social media destination.
  """
  def __init__(self, html, encoding="UTF-8"):
    self.html = html or None
    self.parser = lxml.html.HTMLParser(encoding=encoding)
    self.document = lxml.html.fromstring(html, parser=self.parser)
    self._jsonld_xpath = lxml.etree.XPath('descendant-or-self::script[@type="application/ld+json"]')
    self._metatag_xpath = lxml.etree.XPath("//meta")

  def _extract_items(self, get_item, xpath):
    items = [item for item in map(get_item, xpath(self.document)) if item]
    return dict(ChainMap(*items))

  def _get_metatag_item(self, node):
    name = node.xpath('@property' or '@itemprop' or '@name')
    content = node.xpath('@content')

    return {name[0]: content[0]} \
      if (name and content) else None

  def _get_jsonld_item(self, node):
    return json.loads(node.xpath('string()'))

  def extract_title(self):
    title = self.document.xpath("(//title)[1]//text()")
    return title[0] if len(title) else None

  def extract(self):
    self.metatags = self._extract_items(self._get_metatag_item, self._metatag_xpath)
    self.jsonld = self._extract_items(self._get_jsonld_item, self._jsonld_xpath)
    self.title = self.extract_title()
    return